@@ -1,5 +1,4 @@
 from ..utils import utils
-from .csip_access import ServiceClient
 from .csip_access import csip_worker
 from pyswarms.single.global_best import GlobalBestPSO
 from os import path
@@ -9,9 +8,8 @@ import copy
 import datetime
 import queue
 
-req_queue = queue.Queue()
 
-def service1(x, param, url, step_param_names, step_objfunc, calib_params, full_trace, files=None):
+def eval_cost(x, step_param_names, step_objfunc, calib_params, req_queue):
     cost = np.ones(len(x[:, 0]))
     res_queue = queue.Queue()
 
@@ -32,28 +30,8 @@ def service1(x, param, url, step_param_names, step_objfunc, calib_params, full_t
     return cost
 
 
-def service(x, param, url, step_param_names, step_objfunc, calib_params, full_trace, files=None):
-    cost = np.ones(len(x[:, 0]))
-    threads = []
-    print('   ', end='', flush=True)
-
-    for i_particle, v in enumerate(x[:, 0]):
-        worker = ServiceClient(url, i_particle, cost, x, param,
-                               step_param_names, step_objfunc, calib_params, files, full_trace)
-        threads.append(worker)
-        worker.start()
-        # stagger the calls a bit (250ms)
-        # time.sleep(0.25)
-
-    for x in threads:
-        x.join()
-
-    print(flush=True)
-    return cost
-
-
 def global_best(steps, rounds, args, n_particles, iters, options, n_threads=4, rtol=0.001, ftol=-np.inf,
-                full_trace=None, old=True):
+                full_trace=None):
     """Performs a stepwise particle swarm optimization
 
         Parameters
@@ -70,6 +48,8 @@ def global_best(steps, rounds, args, n_particles, iters, options, n_threads=4, r
             number of iterations
         options : dict
             PSO options (see pyswarms)
+        n_threads : int
+            size of thread pool (default: 4)
         rtol : float
             percentage of change of sum(best_cost) between rounds for
             convergence. Default is :code:`0.001` (0.1 percent)
@@ -108,13 +88,15 @@ def global_best(steps, rounds, args, n_particles, iters, options, n_threads=4, r
     # best round cost
     best_round_cost = 1000
 
+    req_queue = queue.Queue()
+
     done = False
-    workers = []
+    thread_pool = []
     for thread_no in range(n_threads):
-        w = Thread(target=csip_worker, args=(req_queue, thread_no, lambda: done,
+        worker = Thread(target=csip_worker, args=(req_queue, thread_no, lambda: done,
                                              full_trace, args['url'], args['files'], args['param']))
-        workers.append(w)
-        w.start()
+        thread_pool.append(worker)
+        worker.start()
 
     start_time = datetime.datetime.now()
     for r in range(max_rounds):
@@ -132,7 +114,8 @@ def global_best(steps, rounds, args, n_particles, iters, options, n_threads=4, r
             args['step_objfunc'] = objfunc
             # get calibrated parameter from all other steps
             args['calib_params'] = utils.get_calibrated_params(steps, s)
-            args['full_trace'] = full_trace
+
+            args['req_queue'] = req_queue
 
             # create optimizer in the first round.
             if optimizer[s] is None:
@@ -142,11 +125,7 @@ def global_best(steps, rounds, args, n_particles, iters, options, n_threads=4, r
                                                                                          args['calib_params']))
 
             # perform optimization
-            if old:
-                cost, pos = optimizer[s].optimize(service, iters=iters, **args)
-            else:
-                cost, pos = optimizer[s].optimize(service1, iters=iters, **args)
-
+            cost, pos = optimizer[s].optimize(eval_cost, iters=iters, **args)
 
             # capture the best cost
             # if cost < best_cost[s] and np.abs(cost - best_cost[s]) > rtol:
@@ -185,8 +164,8 @@ def global_best(steps, rounds, args, n_particles, iters, options, n_threads=4, r
     print('Done in {} after {} out of {} rounds'.format(elapsed, r + 1, max_rounds))
 
     done = True
-    for w in workers:
-        w.join()
+    for worker in thread_pool:
+        worker.join()
 
     step_trace['rounds'] = r + 1
     step_trace['steps'] = len(steps)
